#if !defined( ISAMFILE_H_INCLUDED ) //{
#   define ISAMFILE_H_INCLUDED

#include "isamdefs.h"   // boolean

enum ISAMError {
        OK,                 // general codes
        SYSTEM_ERROR,
        ILLEGAL_ARG,
        MEM_CANT_ALLOC,
        BAD_CUSTOM_SORT,
        LOCK_WAIT_TIME_OUT,
        OBJECTS_NOT_SUPPORTED,

        FILE_BAD_NAME,      // file codes
        FILE_NAME_TOO_LONG,
        FILE_NOT_OPEN,
        FILE_CANT_OPEN,
        FILE_IS_OPEN,
        FILE_TOO_MANY_OPEN,
        FILE_BAD_FORMAT,
        FILE_EOF,
        FILE_BOF,
        FILE_NON_EXCL_ACCESS,
        FILE_LOCKED,
        FILE_CANT_CREATE_LOCKED,
        FILE_LOCK_TABLE_FULL,

        REC_NOT_FOUND,      // record codes
        REC_DUPLICATE,
        REC_BAD,
        REC_LOCKED,
        REC_NO_CURRENT,
        REC_LOCK_TABLE_FULL,

        IDX_SEGS_2_MANY,    // key and index codes
        IDX_CUR_IS_BAD,
        IDX_CANT_CREATE,
        IDX_ILLEGAL_KEY_DESC,
        IDX_KEY_EXISTS,
        IDX_BAD_KEY,
        IDX_KEY_IS_PRIMARY,

        TRANS_NOT_ON,       // transaction mgmt. codes

        UNKNOWN_ERROR,
};

class ISAMFile {
public:

/* Nested Type Declarations */

    typedef int     AccessID;       // used for build, open, close
    typedef long    CountType;      // used for counting num recs

    enum { MAX_KEY_PARTS = 16 };    // 16 is specific to ISAM engine
                                    // used in current implementation

    enum RecFieldTypes {
            CHAR_TYPE,
            VARCHAR_TYPE,       // variable length, not supported
            INT_TYPE,
            LONG_TYPE,
            DOUBLE_TYPE,
            FLOAT_TYPE,
            OBJECT_TYPE,        // not supported
            USER_DEF_TYPE,      // not supported
    };

    enum LockModes {
            automatic,          // auto lock recs as they're accessed
            manual,             // leave rec locking to manual operation
            exclusive,          // lock all of file
     };

    enum AccessModes {
            read,               // read only
            write,              // write only
            readWrite,
    };

    enum SearchModes {
            first,              // sequential modes
            last,
            next,
            prev,
            current,
            equal,              // random access modes
            greater,
            eqOrGreater,
    };

    enum IndexKeyFlags {
            noDuplicates = 0,       // for entire key
            duplicates = 1,
            noChange = 0x40,
            descending = 0x80,      // for individual key parts only
            noCase = 0x100,
    };

     struct KeyField {
            short start;                // byte position within rec
            short len;                  // length of field in bytes
            int   flags;                // descending or noCase
            RecFieldTypes type;         // data type
    };

    struct IndexKey {
            short flags;        // duplicates, noDuplicates, or noChange
            short nparts;        // number of fields in the key
            KeyField    parts[MAX_KEY_PARTS];
     };

/*@Constructors */

    ISAMFile();
    ISAMFile( const char * file_name );
    ~ISAMFile();

/*@Selectors */

    int     GetFileName( char * buf, int buf_size );

    int     GetRecSize( AccessID file_id );
    int     GetNumKeys( AccessID file_id );
    int     GetIndexSize( AccessID file_id );
    CountType   GetNumRecs( AccessID file_id );
    CountType   GetCurRecNum( AccessID reserved_for_future_use );

    ISAMError GetIndexKey( AccessID file_id,
                                struct IndexKey & info_buf,
                                int index_num );

    int     GetTransNestLevel();

/*@Modifiers */

    // file manipulation
    // =================

    ISAMError  SetFileName( const char * file_name );
    ISAMError  Delete( const char * fileName );
    ISAMError  Rename( const char * newName );

    AccessID      Build( IndexKey * primary_key, int rec_size,
                           ISAMError * error = 0,
                           LockModes lock_mode = exclusive,
                           AccessModes access_mode = readWrite );
    AccessID      Open( ISAMError * error = 0 ,
                          LockModes lock_mode = exclusive,
                          AccessModes access_mode = readWrite );
    ISAMError  Close( AccessID file_id );
    ISAMError  Flush( AccessID file_id );

    // index manipulation
    // ==================

    ISAMError AddIndex(    AccessID file_id, struct IndexKey & new_index );
    ISAMError DelIndex(    AccessID file_id, struct IndexKey & old_index );
    ISAMError SelectIndex( AccessID file_id, struct IndexKey & index,
                                SearchModes mode, char * start_rec,
                                int significant_key_bytes );
    // read/write
    // ==========

    ISAMError ReadEq(   AccessID file_id, char * record, boolean lock = FALSE );
    ISAMError ReadEqGt( AccessID file_id, char * record, boolean lock = FALSE  );
    ISAMError ReadGt(   AccessID file_id, char * record, boolean lock = FALSE  );
    ISAMError ReadNext( AccessID file_id, char * record, boolean lock = FALSE  );
    ISAMError ReadPrev( AccessID file_id, char * record, boolean lock = FALSE  );
    ISAMError ReadFirst(AccessID file_id, char * record, boolean lock = FALSE  );
    ISAMError ReadLast( AccessID file_id, char * record, boolean lock = FALSE  );
    ISAMError ReadCur(  AccessID file_id, char * record, boolean lock = FALSE  );

    ISAMError AddCur(   AccessID file_id, char * record );
    ISAMError Add(      AccessID file_id, char * record );
    ISAMError DeleteCur(AccessID file_id );
    ISAMError Delete(   AccessID file_id, char * record );
    ISAMError Delete(   AccessID file_id, long rec_num );
    ISAMError UpdateCur(AccessID file_id, char * record );             // updates the current record
    ISAMError Update(   AccessID file_id, char * record, long rec_num );  // updates according to rec num
    ISAMError Update(   AccessID file_id, char * record );                // updates according to primary key

    // transaction management
    // ======================

    ISAMError BeginTrans( int * nest_level = 0 );
    ISAMError EndTrans( int * nest_level = 0 );
    ISAMError CancelTrans( int * nest_level = 0 );

    // lock management
    // ===============

    ISAMError LockFile( AccessID file_id );
    ISAMError UnLockFile( AccessID file_id );
    ISAMError UnLockAllRecs( AccessID file_id );

private:

    String          fileName_;
    static int      sTransNestLevel_;   // static because transaction nesting
                                        // levels are a system-wide resource

/* Static Interface */

public:
    static boolean  IsFileIDValid( AccessID file_id );
    static void     InvalidateID( AccessID * file_id );
    static int GetErrString( char * buf, const char * format,
                             ISAMError error_code );

};

#endif //} #if !defined( ISAMFILE_H_INCLUDED ) //{

