// matrix.cpp (listing 2) - demonstrates using the matrix class

#include <iostream>
#include <iomanip>
#include <complex>

#include "matrix.h"
#include "poly.h"

using namespace std;

template<class T>
ostream& operator<<( ostream& o, const matrix<T>& M )
{
   o << endl;
   for(int i=0;i<M.rows;i++) {
     o << "[ ";
     for(int j=0;j<M.cols;j++)
        o << M(i,j) << " ";
     o << "]" << endl;
   }
   o << endl;
   return o;
}

void test1()
{
   cout << "solve the system of linear equations:" << endl;
   cout << endl;
   cout << "  x +       y +       z =  1+i" << endl;
   cout << "  x - (2-3i)y +     3iz = -2+4i" << endl;
   cout << " 3x +      2y - (5+4i)z =  2+3i" << endl;
   cout << endl;

   typedef complex<double> T;

   matrix<T> D(3,3);
   D(0,0) = T(1);   D(0,1) = T(1);    D(0,2) = T(1);
   D(1,0) = T(1);   D(1,1) = T(-2,3); D(1,2) = T(0,3);
   D(2,0) = T(3,0); D(2,1) = T(2);    D(2,2) = T(-5,-4);

   matrix<T> N(3,1);
   N(0,0) = T(1,1);
   N(1,0) = T(-2,4);
   N(2,0) = T(2,3);

   cout << " D = " << D
        << " N = " << N
        << " N.rightdiv(D) = " << N.rightdiv(D) << endl;
}


void test2()
{
   cout << "demonstrate calculating an inverse of a matrix" << endl;

   matrix<long double> A(5,5);

   // initialize matrix to random values
   for(unsigned i=0;i<A.rows;i++)
     for(unsigned j=0;j<A.cols;j++)
        A(i,j) = double(
                   static_cast<double>(rand())/RAND_MAX
                 );

   cout << "A = " << A;

   // create multiplicative inverse
   matrix<long double> Ai(A.inverse());

   cout << " multiplicative inverse Ai = " << Ai;

   // check our work
   cout << "A*Ai = " << A*Ai << endl;
}

void test3()
{
   cout << " solve a system of polynomial equations:" << endl
        << endl
        << "   (x + 1)X   -   (x-1) X     =   4x " << endl
        << "           1             2" << endl
        << "   (x)X       -   (x-1) X     =   3x - 1" << endl
        << "       1                 2" << endl
        << endl
        << "  the solution should be:" << endl
        << "   X   = x+1   X   = x-1" << endl
        << "    1           2 " << endl
        << endl;

   matrix< polynomial<double> > C(2,2);
   matrix< polynomial<double> > N(2,1);

   // x+1
   C(0,0)[1] = 1;
   C(0,0)[0] = 1;

   // -(x-1)
   C(0,1)[1] = -1;
   C(0,1)[0] = 1;

   //  4x
   N(0,0)[1] = 4;

   //  x
   C(1,0)[1] = 1;

   //  -(x-1)
   C(1,1)[1] = -1;
   C(1,1)[0] = 1;

   //  3x - 1
   N(1,0)[1] = 3;
   N(1,0)[0] = -1;

   cout << "C = " << C;
   cout << "N = " << N;
   // solve
   cout << "N.rightdiv(C) = " << N.rightdiv(C);
}

void test4()
{
   matrix< polynomial<double> > M(4,4);

   M(0,0) = 4; M(0,1) = 2;   M(0,2) = -2; M(0,3) = 9;
   M(1,0) = 7; M(1,1) = -3;  M(1,2) = -8, M(1,3) = 5;
   M(2,0) = 0; M(2,1) = 5;  M(2,2) = -8, M(2,3) = -1;
   M(3,0) = 9; M(3,1) = 4;  M(3,2) = -4, M(3,3) = 0;

   cout << "to find the eigenvalues of the matrix " << M;

   polynomial<double> X;
   X[1] = 1;

   cout << "you must find the roots of the equation:" << endl
        << (X = ( M.identity()*X - M ).det()) << " = 0" << endl;
}

void run( void (*f)() )
{
   try {
      f();
   } catch(exception& x) {
      cout << x.what() << endl;
   }
   cout << endl;
}

int main(void)
{
   run(test1);
   run(test2);
   run(test3);
   run(test4);
   return 0;
}
