#if !defined( OBJFILE_H_INCLUDED ) //{
#   define OBJFILE_H_INCLUDED

#include "isamfile.h"

    /*******************************/
    /* Auxiliary Type declarations */
    /*******************************/

struct ObjKeyField {
        int         fieldNum;   // holds enum values
        short       flags;      // same as in ISAMFile
};

struct ObjIndexKey {
        short flags;            // same as in ISAMFile
        short nfields;
        ObjKeyField fields[ ISAMFile::MAX_KEY_PARTS ];
};

struct ObjFieldInfo {
        int     start;
        int     len;
        ISAMFile::RecFieldTypes type;
};


template <class T> class ObjFileIter;       // iterator class

template < class ObjType >
class ObjFile : public ISAMFile {

    friend class ObjFileIter<ObjType>;

public:

/* Selectors */

    boolean IsOpen();

    int             GetNumIndexes();
    CountType       GetNumObjects();
    CountType       GetCurObjNum();

    ISAMError  GetIndexKey( ObjIndexKey & key_buf, int index_num = -1 );

/* Modifiers */

    // file manipulation
    // =================

    ISAMError Build( ISAMFile::LockModes lock_mode = ISAMFile::exclusive,
                          ISAMFile::AccessModes access_mode = ISAMFile::readWrite );
    ISAMError Open( ISAMFile::LockModes lock_mode = ISAMFile::exclusive,
                         ISAMFile::AccessModes access_mode = ISAMFile::readWrite );
    ISAMError Close();
    ISAMError Flush();         // flushes write buffers to disk


    // index manipulation
    // ==================

    ISAMError AddIndex( ObjIndexKey & index_key );
    ISAMError DelIndex( ObjIndexKey & index_key );
    ISAMError SelectIndex( ObjIndexKey & index_key,
                                ObjType * search_obj = NULL,
                                ISAMFile::SearchModes search_mode =
                                                        ISAMFile::equal );
    // read/write
    // ==========

    ISAMError GetCur( ObjType & obj, boolean lock = FALSE );

        /* these change the current position
        */
    ISAMError GetFirst( ObjType & obj, boolean lock = FALSE );
    ISAMError GetLast(  ObjType & obj, boolean lock = FALSE );
    ISAMError GetNext(  ObjType & obj, boolean lock = FALSE );
    ISAMError GetPrev(  ObjType & obj, boolean lock = FALSE );
    ISAMError GetEq(    ObjType & obj, boolean lock = FALSE );
    ISAMError GetEqGt(  ObjType & obj, boolean lock = FALSE );
    ISAMError GetGt(    ObjType & obj, boolean lock = FALSE );

    ISAMError Goto( ObjType & obj );    // >= according to cur index

    ISAMError WriteCur(   ObjType & obj );
    ISAMError Write(      ObjType & obj );
    ISAMError ReWriteCur( ObjType & obj );
    ISAMError ReWrite(    ObjType & obj, CountType obj_num );
    ISAMError ReWrite(    ObjType & obj );
    ISAMError DeleteCur();
    ISAMError DeleteObj(  ObjType & obj );
    ISAMError DeleteObj( CountType obj_num );

    // lock management
    // ===============

    ISAMError LockFile();
    ISAMError UnLockFile();
    ISAMError UnLockAllObjects();

protected:

/* Protected Constructors */
    ObjFile( const char * file_name, ObjFieldInfo * field_info, int num_fields );
    ~ObjFile();

/* Protected Methods */
    virtual int ConvertKey( ObjIndexKey & obj_key, ISAMFile::IndexKey & isam_key );
    virtual int ConvertKey( ISAMFile::IndexKey & isam_key, ObjIndexKey & obj_key );

    virtual boolean IsNull( const ObjType & obj ) const;
    virtual void MakeNull( ObjType & obj );

private:

/* Private Implementation Methods */

    CountType GetCurObjNum( AccessID file_id );

    // file manipulation
    // =================

    AccessID Open( ISAMError * error,
                    ISAMFile::LockModes lock_mode,
                    ISAMFile::AccessModes access_mode );
    ISAMError Close( ISAMFile::AccessID file_id );
    ISAMError Flush( ISAMFile::AccessID file_id );

    // index manipulation
    // ==================

    ISAMError GetIndexKey( AccessID file_id, ObjIndexKey & key_buf, int index_num );
    ISAMError AddIndex( AccessID file_id, ObjIndexKey & index_key );
    ISAMError DelIndex( AccessID file_id, ObjIndexKey & index_key );
    ISAMError SelectIndex( AccessID file_id,
                                ObjIndexKey & index_key,
                                ObjType * search_obj,
                                ISAMFile::SearchModes search_mode );
    // read/write
    // ==========

    ISAMError GetCur(   AccessID file_id, ObjType & obj, boolean lock );
    ISAMError GetFirst( AccessID file_id, ObjType & obj, boolean lock );
    ISAMError GetLast(  AccessID file_id, ObjType & obj, boolean lock );
    ISAMError GetNext(  AccessID file_id, ObjType & obj, boolean lock );
    ISAMError GetPrev(  AccessID file_id, ObjType & obj, boolean lock );
    ISAMError GetEq(    AccessID file_id, ObjType & obj, boolean lock );
    ISAMError GetEqGt(  AccessID file_id, ObjType & obj, boolean lock );
    ISAMError GetGt(    AccessID file_id, ObjType & obj, boolean lock );
    ISAMError Goto(     AccessID file_id, ObjType & obj );
    ISAMError WriteCur(   AccessID file_id, ObjType & obj );
    ISAMError Write(      AccessID file_id, ObjType & obj );
    ISAMError ReWriteCur( AccessID file_id, ObjType & obj );
    ISAMError ReWrite(    AccessID file_id, ObjType & obj, CountType obj_num );
    ISAMError ReWrite(    AccessID file_id, ObjType & obj );
    ISAMError DeleteCur(  AccessID file_id );
    ISAMError DeleteObj(  AccessID file_id, ObjType & obj );
    ISAMError DeleteObj(  AccessID file_id, CountType obj_num );

/* Private Utility Methods */

    virtual int ConvObjToRec( ObjType & obj, char * rec, ObjIndexKey * key = 0 ) = 0;
    virtual int ConvRecToObj( char * rec, ObjType & obj, ObjIndexKey * key = 0 ) = 0;

/* Data Members */

    ISAMFile::AccessID  fileId_;
    ObjIndexKey         curIndex_;

    ObjFieldInfo *  fieldInfo_; // to convert keys to/from isamRec_
    int             numFields_;
    int             recSize_;
    char *          isamRec_;   // working buffer

/* Static Interface */

public:
    static ObjIndexKey &    GetPrimaryKey();
protected:
    static int  SetPrimaryKey( ObjIndexKey & primary_key );
private:
    static ObjIndexKey      sPrimaryKey_; // if none, primaryKey.nfields == 0


};

#endif //} #if !defined( OBJFILE_H_INCLUDED )
