#include "playevnt.h"
#include "isamdefs.h"   // String
#include "bitutils.h"   // LoByte(), etc.
#include <stdlib.h>     // atoi(), etc.
#include <ctype.h>      // isspace()
#include <stdio.h>  // sprintf

char * statStrings[] = {
    "Not Available",
    "Available",
    "Cued",
    "Playing",
    "Paused",
    "Cancelled",
    "Played OK",
    "Played Late",
    "Played Early",
    "Play Error",
};

char * abrevStatStrings[] = {
    "NOTAV",
    "AVAIL",
    "CUED",
    "PLAY",
    "PAUSD",
    "CANCL",
    "MOVED",
    "OK",
    "LATE",
    "EARLY",
    "ERROR",
};

int NUM_STAT_STRINGS = sizeof( statStrings ) / sizeof(char *);

    /****************/
    /* Constructors */
    /****************/

PlayEvent::PlayEvent()
:
    seqNum_( 0 ),
    id_( -1 ),
    status_( Avail ),
    length_( 0 ),
    name_( "" )
{
}

PlayEvent::PlayEvent( IDType new_id )
:
    seqNum_( 0 ),
    id_( new_id ),
    status_( Avail ),
    length_( 0 ),
    name_( "" )
{
}

PlayEvent::~PlayEvent()
{
}

    /*************/
    /* Selectors */
    /*************/

PlayEvent::SeqNumType
PlayEvent::seqNum() const  { return seqNum_; }

PlayEvent::IDType
PlayEvent::id() const  { return id_; }

PlayEvent::PEStatus
PlayEvent::status() const  { return status_; }

int
PlayEvent::length() const { return length_; }

const String &
PlayEvent::name() const    { return name_; }

/**********
    This method returns a string representation of any of this object's
    fields which are listed in the Fields enumeration. The flags parameter
    specifies any appropriate styles or formats according to enum Formats.

    This method uses strncpy(), so if exactly 'size' characters are copied,
    the string will not be null terminated.  If ALL_UPPER or ALL_LOWER is
    specified, then field_buf should be null terminated.  When using these
    flags, the caller should ensure either that a null exists at
    field_buf[size], or that 'size' is large enough.

PARAMETERS:
    field_buf   char *, pointer to buffer into which to place the string
    fields      enum Fields, an enum value specifying the field to
                stringize
    size        int, the max number of characters to put into buf
    flags       enum Formats, format specifications particular to
                each specified field, as follows:

                SeqNum only:
                    default     - the sequence number is returned as a
                                  displayable string
                    BINARY      - the sequence number is placed in binary
                                  format into 4 chars, high order char on
                                  the left.
                Len only:
                    default     - show time in MM::SS format
                    NO_MINS     - show time as seconds only

                Status only:
                    default     - show as a full word, e.g. Available
                    ABREV       - show as max of 5 chars, e.g. Avail

                All fields:
                    default     - fields shown in mixed case
                    ALL_UPPER   - show field in all upper case
                    ALL_LOWER   - show field in all lower case
                    PAD_NULLS   - pad with nulls after the last non-white
                                  space character.
************/

char *
PlayEvent::GetFieldStr( char * field_buf, Fields field,
                        int size, Formats flags )
{
    const int size_buf = 31;
    static char buf[size_buf];        // assume 31 is longest return string

    if ( NULL == field_buf )
        return field_buf;

    switch ( field )
    {
    case SeqNum:
        if ( LoByte( flags ) & BINARY )
        {
            buf[0] = HiByte( HiWord( seqNum_ ));
            buf[1] = LoByte( HiWord( seqNum_ ));
            buf[2] = HiByte( LoWord( seqNum_ ));
            buf[3] = LoByte( LoWord( seqNum_ ));
            memmove( (void *) field_buf, (void *) buf, Min( size, 4 ));
        }
        else
        {
            ltoa( seqNum_, buf, 10 );
            strncpy( field_buf, buf, size );
          }
        break;

    case ID:
        if ( LoByte( flags ) & BINARY )
        {
            buf[0] = HiByte( HiWord( id_ ));
            buf[1] = LoByte( HiWord( id_ ));
            memmove( (void *) field_buf, (void *) buf, Min( size, 2 ));
        }
        else
        {
            itoa( id_, buf, 10 );
            strncpy( field_buf, buf, size );
        }
        break;

    case Status:
        if (( status_ >= 0 ) && ( status_ <= PlayError ))
            if ( LoByte( flags ) & ABREV )      // is abrev
                strncpy( field_buf, abrevStatStrings[ status_ ], size );
            else
                strncpy( field_buf, statStrings[ status_ ], size );
        break;

    case Len:
        if ( LoByte( flags ) & NO_MINS )
            itoa( length_, buf, 10 );
        else
            sprintf( buf, "%.2d:%02.2d", length_ / 60, length_ % 60 );
        strncpy( field_buf, buf, size );
        break;

    case Name:
        strncpy( field_buf, name_.gets(), size );
        break;

    default:
        break;

    }       /* switch ( field ) */

    if ( flags & PAD_NULLS )
    {
        char * ptr = field_buf + Min( size, size_buf ) - 1;
        while ( isspace( *ptr ))
            *ptr-- = '\0';
    }

    if ( flags & ALL_UPPER )
        strupr( field_buf );
    else if ( flags & ALL_LOWER )
        strlwr( field_buf );

    return field_buf;

}       /* PlayEvent::GetFieldStr( char *, Fields, int, int ) */


    /*************/
    /* Modifiers */
    /*************/

int
PlayEvent::seqNum( SeqNumType new_seq_num )
{
    seqNum_ = new_seq_num;
    return 0;
}

int
PlayEvent::id( int new_id )  { id_ = new_id; return 0; }

int
PlayEvent::status( PEStatus new_status )
{
    status_ = new_status;
    return 0;
}

int
PlayEvent::length( int new_len )    { length_ = new_len; return 0; }

int
PlayEvent::name( const String & new_name )
{
    name_.puts( new_name.gets() );
    return 0;
}

/**********
    This method sets the fields in this object according to the string
    representation specified.  This method parses and validates the string,
    returning any of StrErrors.

    The expected formats of new_val are as follows (where ? is any
    character, * is any substring, n is any numeric character):

    ID:         nnn*, any number of n's

    SeqNum:     if flags specifies BINARY, then four chars with the high
                order char on the left, otherwise nnn*, any number of n's.

    Len:        if flags specifies NO_MINS, then nnn*, otherwise nn?nn*

    Status:     SSS*, where SSS is any of the strings in the array
                abrevStatStrings in the *.cpp file for this class.

    All others: *, simply copied, not parsed.

PARAMETERS:
    new_val     const char *, pointer to string containing new value
    field       enum Fields, enum specifying which field to update.
    size        int, maximum number of characters to read from new_val, if
                0 then read all of them.
    flags       enum Formats, enum specifying format in which this string
                is coming in.

RETURN VALUE:
    int specifying one of StrErrors.

COMMENTS:
    If parameter new_val is NULL, nothing happens.

************/
int
PlayEvent::PutFieldStr( const char * new_val, Fields field,
                        int size, Formats flags )
{
    int ws_size = 35;
    static char * work_str = new char[ ws_size ];
    int ret = OK;

    if ( NULL == new_val )
        return BAD_VALUE;

        /* Make sure that work buffer is large enough, reallocate if not.
        ****/
    if ( size >= ws_size )
    {
        delete [] work_str;
        ws_size = size + 1;                 // need room for null terminator
        work_str = new char[ ws_size ];
        work_str[ ws_size - 1 ] = '\0';     // put null terminator
    }

        /* If size == 0, use new_val intact, else copy new_val into buffer,
        append null terminator, and use that buffer.
        ****/
    char * val_ptr;
    if ( size == 0 )
        val_ptr = (char *) new_val;
    else
    {
        strncpy( work_str, new_val, size );
        work_str[size] = '\0';
        val_ptr = work_str;
    }

    switch ( field )
    {
    case SeqNum:
        if ( size < 4 )
            break;
        if ( LoByte( flags ) & BINARY )
            seqNum_ = MakeLong( MakeWord( new_val[3], new_val[2] ),
                               MakeWord( new_val[1], new_val[0] ));
        else
            seqNum_ = atol( new_val );
        break;

    case ID:
        if ( size < 2 )
            break;
        if ( LoByte( flags ) & BINARY )
            id_ = MakeWord( new_val[1], new_val[0] );
        else
            id_ = atoi( val_ptr );
        break;

    case Status:
        {
        int i;
        for ( i = 0; i <= NUM_STAT_STRINGS; i++ )
            if ( 0 == stricmp( val_ptr, abrevStatStrings[i] ))
                break;

        if ( i <= PlayError )
            status_ = (PEStatus) i;
        else
            ret = BAD_VALUE;
        }
        break;

    case Len:
        if ( LoByte( flags ) & NO_MINS )
            length_ = atoi( val_ptr );
        else
        {
            int lm = 0, ls = 0;
            sscanf( val_ptr, "%2d%*c%2d", &lm, &ls );
            length_ = lm*60 + ls;
        }
        break;

    case Name:
        name_.puts( val_ptr );
        break;

    default:
        ret = BAD_VALUE;

    }       /* switch ( field ) */

    return ret;

}       /* PlayEvent::PutFieldStr( const char *, Fields, int, int ) */
