// poly.h (listing 3) - polynomial template class
//
#include <stdexcept>
#include <vector>
#include <memory>
#include <function>
#include <stdlib>

template<class T>
class polynomial {
   public:
      ~polynomial();
      // constructors
      // initializes the constant term
      polynomial( const T& x0 = T(0.0) );
      // initializes from an array of coefficients
      polynomial( unsigned nc, const T *init );
      // copy constructor
      polynomial( const polynomial<T>& cp );

      // evaluate the function for a given x
      T operator()( const T& x ) const;
      // chain a polynomial
      polynomial<T> operator()( const polynomial<T>& P ) const;

      // access coefficient array
      T& operator[](int);
      const T& operator[](int) const;

      // equality
      bool operator==( const polynomial<T>& A ) const;
      bool operator!() const {
         return *this == polynomial<T>(0.0);
      }

      // assignment
      polynomial<T>& operator=( const polynomial<T>& cp );

      // arithmetic
      polynomial<T>& operator+=( const polynomial<T>& P );
      polynomial<T> operator+( const polynomial<T>& P ) const {
         return polynomial<T>(*this) += P;
      }
      polynomial<T>& operator-=( const polynomial<T>& P );
      polynomial<T> operator-( const polynomial<T>& P ) const {
         return polynomial<T>(*this) -= P;
      }
      polynomial<T> operator-() const;
      polynomial<T> operator+() const {
         return *this;
      }

      // same as multiplying by x ^ i
      polynomial<T> operator<<( int i ) const {
         return polynomial<T>(*this) <<= i;
      }
      // same as multiplying by x ^ -i
      polynomial<T> operator>> ( int i ) const {
         return operator<<(-i);
      }
      polynomial<T>& operator<<=( int i ) {
         ofs-=i;
         return *this;
      }
      polynomial<T>& operator>>=( int i ) {
         return operator<<=(-i);
      }
      // multiplication
      polynomial<T> operator*( const polynomial<T>& P ) const;
      polynomial<T>& operator*=( const polynomial<T>& P ) {
         return *this = (*this * P);
      }
      // division
      polynomial<T> operator/( const polynomial<T>& D ) const;
      polynomial<T>& operator/=( const polynomial<T>& D ) {
         return *this = operator/(D);
      }

      // polynomial raised to a power
      polynomial<T> pow( int exp ) const {
         polynomial P;
         P[exp] = T(1.0);
         return P(*this);
      }


      // returns the deriviative function
      polynomial<T> derivative() const;

      // returns the integral function for the interval from c to x
      polynomial<T> integral( const T& c ) const;

      int upper() const;
      int lower() const;

   private:
      std::vector<T> C; // array of coefficients
      unsigned n;       // number of elements in array
      int ofs;          // array offset
      const T T0;       // additive identity of template parameter
};

template<class T>
polynomial<T>::~polynomial()
{
}

template<class T>
polynomial<T>::polynomial( const T& x0 = T(0.0) )
   : T0(0.0), C(0), n(0), ofs(0)
{
   (*this)[0] = x0;
}

template<class T>
polynomial<T>::polynomial( unsigned nc, const T* init )
   : T0(0.0), C(0), n(0), ofs(0)
{
   for(int i=nc;i>=0;i--)
     (*this)[i] = init[i];
}

template<class T>
polynomial<T>::polynomial( const polynomial<T>& cp )
   : T0(0.0), C(0), n(0), ofs(0)
{
   operator=(cp);
}

template<class T>
T& polynomial<T>::operator[]( int i )
// access coefficient
// this may cause a new element of the array to be created
{
   if(n==0) {
       // no elements yet, create one
       C.insert(C.begin(),1,T0);
       n = 1;
       ofs = -i;
   } else if( i+ofs < 0 ) {
      // need to insert elements at the beginning of the vector
      int num = -i-ofs;
      C.insert(C.begin(),num,T0);
      ofs = -i;
      n += num;
   } else if( n <= i+ofs ) {
      // append new elements to the end of the array
      C.insert(C.end(),i+ofs+1-n,T0);
      n = i+ofs+1;
   }
   return C[i+ofs];
}

template<class T>
const T& polynomial<T>::operator[](int i) const
{
   if( (i+ofs) < 0 || n <= (i+ofs) )
      return T0; // return T0 for out of range elements
   return C[i+ofs];
}

template<class T>
polynomial<T>& polynomial<T>::operator=( const polynomial<T>& cp )
// assignment
{
   if( this == &cp )
      return *this;
   C.empty();
   n = 0;
   ofs = 0;
   for(int i=cp.lower();i<=cp.upper();i++)
      (*this)[i] = cp[i];
   return *this;
}


template<class T>
T polynomial<T>::operator()( const T& x ) const
// evaluation the polynomial function for a given x
{
   T r(0.0);
   for(int i=lower();i<=upper();i++) {
      T p( (*this)[i] );
      if(p != T0) {
         if(i<0) {
            for(int j=0;j<-i;j++)
               p /= x;
         } else if(i>0) {
            for(int j=0;j<i;j++)
               p *= x;
         }
      }
      r += p;
   }
   return r;
}

template<class T>
polynomial<T> polynomial<T>::operator()( const polynomial<T>& X ) const
// chain the polynomial X
// this is the polynomial that results from  P(X(y)) where P is *this
{
   polynomial<T> R(0.0);
   for(int i=lower();i<=upper();i++) {
      if( (*this)[i] != T0 ) {
         polynomial<T> P( (*this)[i] );
         if(i<0) {
            for(int j=0;j<-i;j++)
              P /= X;
         } else if(i>0) {
            for(int j=0;j<i;j++)
              P *= X;
         }
         R += P;
      }
   }
   return R;
}

template<class T>
polynomial<T>& polynomial<T>::operator+=( const polynomial<T>& P )
{
   for( int i=P.lower();i<=P.upper();i++)
      (*this)[i] += P[i];
   return *this;
}

template<class T>
polynomial<T>& polynomial<T>::operator-=( const polynomial<T>& P )
{
   for( int i=P.lower();i<=P.upper();i++)
      (*this)[i] -= P[i];
   return *this;
}

template<class T>
bool polynomial<T>::operator==( const polynomial<T>& P ) const
{
   int mx = max(upper(),P.upper());
   int mn = min(lower(),P.lower());
   for(int i=mn;i<=mx;i++)
      if( (*this)[i] != P[i] )
         return false;
   return true;
}

template<class T>
polynomial<T> polynomial<T>::operator-() const
// unary -
{
   polynomial<T> r;
   for(int i=lower();i<=upper();i++)
      r[i] = -(*this)[i];
   return r;
}

template<class T>
int polynomial<T>::upper() const
// returns highest non-zero x coefficient
{
   for(int i=n-1;i>=0;i--)
      if( C[i] != T0 )
         return i-ofs;
   return 0;
}

template<class T>
int polynomial<T>::lower() const
// returns lowest non-zero x coefficient
{
   for(int i=0;i<n;i++)
      if( C[i] != T0 )
         return i-ofs;
   return 0;
}

template<class T>
inline
polynomial<T> operator*( const T& A, const polynomial<T>& B )
{
   return B*polynomial<T>(A);
}

template<class T>
polynomial<T> polynomial<T>::operator*( const polynomial<T>& B ) const
// polynomial multiplication
{
   const polynomial<T>& A = *this;
   polynomial<T> r;
   for(int i=A.lower();i<=A.upper();i++) {
      polynomial<T> p(B);
      for(int j=B.lower();j<=B.upper();j++)
         p[j] *= A[i];
      r += ( p << i );
   }
   return r;
}

template<class T>
inline
polynomial<T> operator/( const T& N, const polynomial<T> D )
{
   return polynomial<T>(N)/D;
}

template<class T>
polynomial<T> polynomial<T>::operator/( const polynomial<T>& D ) const
// polynomial division
//
// The result is a polynomial function, which means that it is possible
// the the represented polynomial is not completely represented because
// of repeating terms.
//
// For instance,  1/(x+1) is the repeating polynomial:
//
//     -1    -2     -3    -4    -5             (n+1)  -n
//    x   - x   + x    - x   + x   + ... + (-1)     x    + ...
//
// With this implementation, it is carried out to 100 terms beyond the
// lowest term of the numerator or the denominator.
//
// For values of x where |x| <= 1, this can cause the resulting polynomial
// to not accurately represent the actual polynomial.
//
{
   const polynomial<T>& N = *this;
   polynomial<T> Q;     // quotient
   polynomial<T> R(N);  // remainder (start with numerator)
   const polynomial<T> p0;  // additive identity polynomial

   const int nu = N.upper();
   const int l = min(N.lower(),D.lower());
   const int du = D.upper();

   // note - only 100 extra terms will be carried out
   for( int i = nu; i>= l-100; i--) {
      T q = R[i]/D[du];
      Q[i-du] = q;
      R -= ( ( D * q ) << (i-du) );
      R[i] = T0;
      if( R == p0 ) // if remainder is zero, then work is done
         break;
   }
   return Q;
}

template<class T>
polynomial<T> polynomial<T>::derivative() const
// calculate the derivative of the polynomial
//                                    n        n-1
// derivative of a polynomial term  cx   is cnx
//
//                                             0
// with the exception of the constant term or x , which is zero
//
{
   polynomial<T> D(*this);
   for(int i=D.lower();i<=D.upper();i++)
      if(i==0)
        D[i] = T0;  // derivative is zero for the constant term
      else
        D[i] *= T(double(i));
   D >>= 1; // divide polynomial by x
   return D;
}

template<class T>
polynomial<T> polynomial<T>::integral( const T& c ) const
// calculate the polynomial integral function
// note:  if the polynomial has a non-zero 1/x term, this function
// will throw a domain_error, since the integral function cannot be
// represented by a polynomial equation.
//                                n                     n+1
// integral function for a term cx  then I =  (c/(n+1))x    + C
//
{
   polynomial<T> I(*this); // make a copy
   int lowest = 0;
   for(int i=I.lower();i<=I.upper();i++) {
      if( I[i] != T0 ) {
         if( i<1 && !lowest )
            lowest = i;
         if(i+1 == 0)
            throw std::domain_error("integral function is not a polynomial");
         I[i] /= T(double(i+1));
      }
   }
   I <<= 1;  // multiply entire function by x
   I[0] = -I(c);  // calculate constant term
   return I;
}

template<class T>
ostream& operator<<( ostream& o, const polynomial<T>& p )
// outputs a human readable polynomial
{
   bool showzero = true;
   cout << "( ";
   for(int i=p.upper();i>=p.lower();i--) {
      if(p[i] != T(0.0) ) {
         if( (i==0) || ( p[i] != T(1.0) ) ) {
             o << p[i];
             showzero = false;
         }
         if( i > 1 || i < 0 ) {
            showzero = false;
            o << "x^" << i;
         } else if( i==1) {
            showzero= false;
            o << "x";
         }
         if( i != p.lower() )
            o << "+";
      }
   }
   if(showzero)
      cout << T(0.0);
   cout << " )";
   return o;
}

ostream& operator<<( ostream& o, const polynomial<double>& p )
// specialization for double
{
   bool showzero = true;
   cout << "( ";
   for(int i=p.upper();i>=p.lower();i--) {
      if( p[i] != 0.0 ) {
         if( i != p.upper() && p[i] > 0 )
            o << "+";
         if( (i==0) || ( p[i] != 1.0 ) ) {
             if( p[i] == -1.0 && i )
                o << "-";
             else
                o << p[i];
             showzero = false;
         }
         if( i > 1 || i < 0 ) {
            showzero = false;
            o << "x^" << i;
         } else if( i==1 ) {
            showzero= false;
            o << "x";
         }
      }
   }
   if(showzero)
      cout << 0.0;
   cout << " )";
   return o;
}
